import os
import pickle
import copy
import numpy as np
from scipy.spatial.transform import Rotation
from TB2J.io_exchange import SpinIO

# Rotation from x to z
Rxz = Rotation.from_euler('y', -90, degrees=True)
# Rotation from y to z
Ryz = Rotation.from_euler('x', -90, degrees=True)


def rot_merge_DMI(Dx, Dy, Dz):
    Dx_z = Rxz.apply(Dx)
    Dy_z = Ryz.apply(Dy)
    D = (np.array([0.0, 0.5, 0.5]) * Dx_z + np.array([0.5, 0.0, 0.5]) * Dy_z +
         np.array([0.5, 0.5, 0.0]) * Dz)
    return D


def rot_merge_DMI2(Dx, Dy, Dz):
    Dx_z = Rxz.apply(Dx)
    Dy_z = Ryz.apply(Dy)
    D = (np.array([1, 0, 0]) * Dx_z + np.array([0, 1, 0]) * Dy_z +
         np.array([0, 0, 1]) * Dz)
    return D


def merge_DMI(Dx, Dy, Dz):
    D = (np.array([0.0, 0.5, 0.5]) * Dx + np.array([0.5, 0.0, 0.5]) * Dy +
         np.array([0.5, 0.5, 0.0]) * Dz)
    return D


def merge_DMI2(Dx, Dy, Dz):
    D = (np.array([1, 0, 0]) * Dx + np.array([0, 1, 0]) * Dy +
         np.array([0, 0, 1]) * Dz)
    return D


def swap_direction(m, idirections):
    """
    swap two directions of a tensor m.
    idirections: the index of the two directions.
    """
    idirections = list(idirections)
    inv = [idirections[1], idirections[0]]
    n = np.copy(m)
    n[:, idirections] = n[:, inv]
    n[idirections, :] = n[inv, :]
    return n


def test_swap():
    m = np.reshape(np.arange(9), (3, 3))
    print(m)
    print(swap_direction(m, [0, 1]))


def merge_Jani(Janix, Janiy, Janiz):
    Jani = (np.array([[0, 0, 0], [0, 1, 1], [0, 1, 1]]) * Janix +
            np.array([[1, 0, 1], [0, 0, 0], [1, 0, 1]]) * Janiy +
            np.array([[1, 1, 0], [1, 1, 0], [0, 0, 0]]) * Janiz) / 2.0
    return Jani


def read_pickle(path):
    p1 = os.path.join(path, 'TB2J_results', 'TB2J.pickle')
    p2 = os.path.join(path, 'TB2J.pickle')
    if os.path.exists(p1) and os.path.exists(p2):
        print(f" WARNING!: Both file {p1} and {p2} exist. Use default {p1}.")
    if os.path.exists(p1):
        ret = SpinIO.load_pickle(os.path.join(path, 'TB2J_results'))
    elif os.path.exists(p2):
        ret = SpinIO.load_pickle(path)
    else:
        raise FileNotFoundError(f"Cannot find either file {p1} or {p2}")
    return ret


class Merger():
    def __init__(self, path_x, path_y, path_z, method='structure'):
        assert (method in ['structure', 'spin'])
        self.dat_x = read_pickle(path_x)
        self.dat_y = read_pickle(path_y)
        self.dat_z = read_pickle(path_z)
        self.dat = copy.copy(self.dat_z)
        self.paths = [path_x, path_y, path_z]
        self.method = method

    def merge_Jani(self):
        Jani_dict = {}
        Janixdict = self.dat_x.Jani_dict
        Janiydict = self.dat_y.Jani_dict
        Janizdict = self.dat_z.Jani_dict
        for key, Janiz in Janizdict.items():
            try:
                R, i, j = key
                keyx = R
                keyy = R
                Janix = Janixdict[(tuple(keyx), i, j)]
                Janiy = Janiydict[(tuple(keyy), i, j)]
            except KeyError as err:
                raise KeyError(
                    "Can not find key: %s, Please make sure the three calculations use the same k-mesh and same Rcut."
                    % err)
            if self.method == 'spin':
                Jani_dict[key] = merge_Jani(Janix, Janiy, Janiz)
            else:
                Jani_dict[key] = merge_Jani(swap_direction(Janix, (0, 2)),
                                            swap_direction(Janiy, (1, 2)),
                                            Janiz)
        self.dat.Jani_dict = Jani_dict

    def merge_Jiso(self):
        Jdict={}
        Jxdict=self.dat_x.exchange_Jdict
        Jydict=self.dat_y.exchange_Jdict
        Jzdict=self.dat_z.exchange_Jdict
        for key, J in Jzdict.items():
            try:
               Jx = Jxdict[key]
               Jy = Jydict[key]
               Jz = Jzdict[key]
            except KeyError as err:
                raise KeyError(
                        "Can not find key: %s, Please make sure the three calculations use the same k-mesh and same Rcut."
                        % err)
            Jdict[key]=(Jx+Jy+Jz)/3.0
        self.dat.exchange_Jdict=Jdict
               

    def merge_DMI(self):
        dmi_ddict = {}
        if self.dat_x.has_dmi and self.dat_y.has_dmi and self.dat_z.has_dmi:
            Dxdict = self.dat_x.dmi_ddict
            Dydict = self.dat_y.dmi_ddict
            Dzdict = self.dat_z.dmi_ddict
            for key, Dz in Dzdict.items():
                try:
                    R, i, j = key
                    keyx = R
                    keyy = R
                    Dx = Dxdict[(tuple(keyx), i, j)]
                    Dy = Dydict[(tuple(keyy), i, j)]
                except KeyError as err:
                    raise KeyError(
                        "Can not find key: %s, Please make sure the three calculations use the same k-mesh and same Rcut."
                        % err)
                if self.method == 'structure':
                    dmi_ddict[key] = rot_merge_DMI(Dx, Dy, Dz)
                else:
                    dmi_ddict[key] = merge_DMI(Dx, Dy, Dz)
            self.dat.dmi_ddict = dmi_ddict

        dmi_ddict = {}
        try:
            Dxdict = self.dat_x.debug_dict['DMI2']
            Dydict = self.dat_y.debug_dict['DMI2']
            Dzdict = self.dat_z.debug_dict['DMI2']
            for key, Dz in Dzdict.items():
                try:
                    R, i, j = key
                    keyx = R
                    keyy = R
                    Dx = Dxdict[(tuple(keyx), i, j)]
                    Dy = Dydict[(tuple(keyy), i, j)]
                except KeyError as err:
                    raise KeyError(
                        "Can not find key: %s, Please make sure the three calculations use the same k-mesh and same Rcut."
                        % err)
                if self.method == 'structure':
                    dmi_ddict[key] = rot_merge_DMI2(Dx, Dy, Dz)
                elif self.method == 'spin':
                    dmi_ddict[key] = merge_DMI2(Dx, Dy, Dz)

            self.dat.debug_dict['DMI2'] = dmi_ddict
        except:
            pass

    def write(self, path='TB2J_results'):
        self.dat.description += 'Merged from TB2J results in paths: \n  ' + '\n  '.join(
            self.paths) + '\n'
        if self.method == 'spin':
            self.dat.description += ', which are from DFT data with spin along x, y, z orientation\n'
        elif self.method == 'structure':
            self.dat.description += ', which are from DFT data with structure with z axis rotated to x, y, z\n'
        self.dat.description += '\n'
        self.dat.write_all(path=path)


def merge(path_x, path_y, path_z, method, save=True, path='TB2J_results'):
    m = Merger(path_x, path_y, path_z, method)
    m.merge_Jiso()
    m.merge_DMI()
    m.merge_Jani()
    if save:
        m.write(path=path)
    return m.dat



import numpy as np
import scipy.linalg as sl
from collections import defaultdict
from ase.dft.kpoints import monkhorst_pack
import os
import matplotlib.pyplot as plt


def eigen_to_G(evals, evecs, efermi, energy):
    """ calculate green's function from eigenvalue/eigenvector for energy(e-ef): G(e-ef).
    :param evals:  eigen values
    :param evecs:  eigen vectors
    :param efermi: fermi energy
    :param energy: energy
    :returns: Green's function G,
    :rtype:  Matrix with same shape of the Hamiltonian (and eigenvector)
    """
    return evecs.dot(np.diag(1.0 / (-evals + energy + efermi))).dot(
        evecs.conj().T)


def H2G(H, energy):
    return np.linalg.inv(energy*np.eye(len(H)) - H)


def Gpert(H0, dH, e):
    """
    Show Dyson
    """
    evals, evecs=sl.eigh(H0)
    G0=eigen_to_G(evals, evecs, 0.0, e)

    G01= H2G(H0, e)
    print (G0-G01)

    evals, evecs=sl.eigh(H0+dH)
    G=eigen_to_G(evals, evecs, 0.0, e)

    print(G-(G0@dH@G+G0))
    print(G-(G0@dH@G0+G0))
    #print(G0@dH@G0+G0@dH@G0@dH@G0-G+G0)
    #print(G0@dH@G0+G0@dH@G0@dH@G0+G0@dH@G0@dH@G0@dH@G0-G+G0)

def test():
    H0=np.random.rand(4,4)
    H0=H0+H0.T.conj()

    dH=np.random.rand(4,4)
    dH=H0+H0.T.conj()

    dH=dH*0.1

    Gpert(H0, dH, 1.3+0.00j)


#test()



#!/usr/bin/env python
import math
import numpy as np
import scipy.linalg as linalg

class QSolver(object):
    def __init__(self, hamiltonian):
        self.ham = hamiltonian
        self.nspin = self.ham.nspin
        M = linalg.norm(self.ham.spinat, axis=1)
        self.M_mat=np.kron(np.sqrt(np.einsum('i,j->ij', M, M)), np.ones((3,3)))
        self.Eref=None

    def get_Eref(self):
        mat = np.zeros((3 * self.nspin, 3 * self.nspin), dtype=complex)
        for key, val in self.ham.get_total_hessian_ijR().items():
            i, j, R = key
            mat[i * 3:i * 3 + 3, j * 3:j * 3 + 3] -= val
        mat=mat*4.0/self.M_mat
        SZ=np.zeros(self.nspin*3,dtype=float)
        SZ[2::3]=1.0
        SZ/=np.linalg.norm(SZ)
        self.Eref=np.dot(SZ, np.dot(mat, SZ)).real
        return self.Eref

    def solve_k(self, kpt, eigen_vectors=True, Jq=False):
        if self.Eref is None and not Jq:
            self.get_Eref()
        mat = np.zeros((3 * self.nspin, 3 * self.nspin), dtype=complex)
        for key, val in self.ham.get_total_hessian_ijR().items():
            i, j, R = key
            mat[i * 3:i * 3 + 3, j * 3:j * 3 + 3] -= val * np.exp(
               2.0j * math.pi * np.dot(kpt, R))
        if not Jq:
            mat=mat*4.0/self.M_mat
        if eigen_vectors:
            evals, evecs = linalg.eigh(mat)
            evals=np.real(evals)
            if not Jq:
                evals-=self.Eref
            return evals, evecs
        else:
            evals = np.linalg.eigvalsh(mat)
            if not Jq:
                evals-=self.Eref
            return evals

    def solve_all(self, kpts, eigen_vectors=True, Jq=False):
        eval_list = []
        evec_list = []
        if eigen_vectors:
            for kpt in kpts:
                evals, evecs = self.solve_k(kpt, eigen_vectors=True, Jq=Jq)
                eval_list.append(evals)
                evec_list.append(evecs)
            return np.array(eval_list), np.array(evec_list)
        else:
            for kpt in kpts:
                evals = self.solve_k(kpt, eigen_vectors=False, Jq=Jq)
                eval_list.append(evals)
            return np.array(eval_list)



import os
import matplotlib.pyplot as plt
from TB2J.spinham.spin_api import SpinModel
from TB2J.io_exchange.io_exchange import SpinIO
from TB2J.io_exchange.io_txt import write_Jq_info
from ase.dft.kpoints import monkhorst_pack
from ase.cell import Cell
import numpy as np
from TB2J import __version__

def write_emin(**kwargs):
    m=SpinIO.load_pickle("./")
    m.write_Jq(kmesh=[9, 9, 9], path='./', **kwargs)

def plot_magnon_band(fname='exchange.xml',
                     path='./',
                     npoints=301,
                     show=True,
                     kvectors=None,
                     knames=None,
                     figfname="magnon_band.pdf",
                     Jq=False,
                     kpath_fname='exchange_kpth.txt',
                     **kwargs
                     ):
    m = SpinModel(fname=fname, sc_matrix=None)
    m.set_ham(**kwargs)
    m.plot_magnon_band(kvectors=kvectors,
                       knames=knames,
                       npoints=npoints,
                       kpath_fname=kpath_fname,
                       Jq=Jq,
)

    plt.savefig(figfname)
    if show:
        plt.show()



#!/usr/bin/env python
import numpy as np
from collections import Iterable, defaultdict
import matplotlib.pyplot as plt
from ase.dft.kpoints import bandpath, monkhorst_pack
from .hamiltonian_terms import (ZeemanTerm, UniaxialMCATerm, ExchangeTerm,
                                DMITerm, BilinearTerm)
from .constants import mu_B, gyromagnetic_ratio
from .supercell import SupercellMaker
from .spin_xml import SpinXmlParser, SpinXmlWriter
from .plot import group_band_path
from ase.cell import Cell
#from minimulti.spin.hamiltonian import SpinHamiltonian
#from minimulti.spin.mover import SpinMover
from .qsolver import QSolver


class SpinHamiltonian(object):
    def __init__(self,
                 cell=None,
                 pos=None,
                 xcart=None,
                 spinat=None,
                 zion=None,
                 Rlist=None,
                 iprim=None):
        if xcart is None:
            self.xcart = np.dot(pos, cell)
            self.pos = pos
        elif pos is None:
            self.pos = np.dot(xcart, np.linalg.inv(cell))
            self.xcart = xcart
        self.cell = cell
        self.xcart = pos
        self.spinat = spinat
        self.zion = zion

        self._spin = np.array(spinat) * mu_B
        self._map_to_magnetic_only()
        # spin interaction parameters

        # Homogeneous H eff_field
        self.has_external_hfield = False
        self.zeeman_H = np.zeros(3, dtype='float64')

        # uniaxial single ion anisotropy
        self.has_uniaxial_anisotropy = False

        # cubic Anistropy (Not implemented yet)
        self.has_cubic_anistropy = False

        # Exchange interaction
        self.has_exchange = False
        self.exchange_Jdict = None

        # DM interaction
        self.has_dmi = False
        self.dmi_ddict = None

        # Dipole dipole interaction
        self.has_dipdip = False

        # bilinear term
        self.has_bilinear = False
        self.bilinear_dict = None

        # calculation parameters

        # calculation results

        # hamiltonian list
        self.hamiltonians = {}

        self._total_hessian_ijR = None

        self.gilbert_damping = np.ones(self.nspin) * 1.0
        self.gyro_ratio = np.ones(self.nspin) * gyromagnetic_ratio

        # internal variables:
        self._langevin_tmp = None

        self.sublattice = list(range(self.nspin))

        self.index_spin = None
        self.lattice = None

        self.iprim = iprim
        self.Rlist = Rlist

    def set_lattice(self, atoms, index_spin):
        self.lattice = atoms
        self.index_spin = index_spin

    @property
    def spin_positions(self):
        return self.pos

    def _map_to_magnetic_only(self):
        """
        select the sites with spin and re-index.
        """
        ms = np.linalg.norm(self._spin, axis=1)
        self.magsites = np.where(ms > 0.00001 * mu_B)[0]
        self.ms = ms[self.magsites]
        S = self._spin[self.magsites]
        self.nspin = len(S)
        # self.S = np.array(
        #    [S[i] / self.ms[i] for i in range(self.nspin)],
        #    dtype='float64')
        self.s = S / self.ms[:, None]

    @property
    def spin(self):
        self._spin[self.magsites] = np.multiply(self.ms[:, np.newaxis],
                                                self.s) / mu_B
        return self._spin

    @spin.setter
    def spin(self, spin):
        self._spin = np.array(spin) * mu_B
        self._map_to_magnetic_only()

    def normalize_S(self):
        """
        normalize so the norm of self.S[i] is 1
        """
        snorm = np.linalg.norm(self.s, axis=1)
        self.s /= np.expand_dims(snorm, axis=1)

    def set(self, gilbert_damping=None, gyro_ratio=None, 
            has_exchange=None, has_dmi=None, has_bilinear=None,
            has_external_hfield=None, has_uniaxial_anisotropy=None):
        """
        set parameters for simulation:
        args:
        ====================
        timestep: in ps
        temperature: in K, default 0K.
        damping factor: Gilbert damping factor, default: 0.01
        gyromagnetic_ratio: default 1.0.
        """
        if gilbert_damping is not None:
            self.gilbert_damping = np.array(gilbert_damping)
        if gyro_ratio is not None:
            self.gyro_ratio = np.array(gyro_ratio)
        self.has_exchange=self.has_exchange and bool(has_exchange)
        self.has_dmi=self.has_dmi and bool(has_dmi)
        self.has_bilinear=self.has_bilinear and bool(has_bilinear)
        self.has_external_hfield=self.has_external_hfield and bool(has_external_hfield)
        self.has_uniaxial_anisotropy=self.has_uniaxial_anisotropy and bool(has_uniaxial_anisotropy)

    def set_exchange_ijR(self, exchange_Jdict):
        """
        J: [(i,j, R, J_{ijR})] J_{ijR} is a scalar
        """
        self.has_exchange = True
        self.exchange_Jdict = exchange_Jdict
        exchange = ExchangeTerm(self.exchange_Jdict, ms=self.ms)
        self.hamiltonians['exchange'] = exchange

    def set_dmi_ijR(self, dmi_ddict):
        """
        D: [(i,j, R, D_{ijR})], D_{ijR} is a vector
        """
        self.has_dmi = True
        self.dmi_ddict = dmi_ddict
        DMI = DMITerm(self.dmi_ddict, self.ms)
        self.hamiltonians['DMI'] = DMI

    def set_bilinear_ijR(self, bilinear_dict):
        self.has_bilinear = True
        self.bilinear_dict = bilinear_dict
        Bi = BilinearTerm(self.bilinear_dict, self.ms)
        self.hamiltonians['Bilinear'] = Bi

    def set_dipdip(self):
        """
        add the dipole dipole interaction term.
        """
        pass

    def set_external_hfield(self, H):
        """
        add external magnetic field. If H is a vector , it is homogenoues.
            Otherwise H should be given as a nspin*3 matrix.
        """
        self.has_external_hfield = True
        if isinstance(H, Iterable):
            self.H_ext = np.asarray(H)
        else:
            self.H_ext = np.ones([self.nspin, 3]) * H
        zeeman = ZeemanTerm(H=self.H_ext, ms=self.ms)
        self.hamiltonians['zeeman'] = zeeman

    def set_uniaxial_mca(self, k1, k1dir):
        """
        Add homogenoues uniaxial anisotropy
        """
        self.has_uniaxial_anisotropy = True

        self.k1 = k1
        self.k1dir = k1dir
        umcaterm = UniaxialMCATerm(k1, k1dir, ms=self.ms)
        self.hamiltonians['UMCA'] = umcaterm

    def add_Hamiltonian_term(self, Hamiltonian_term, name=None):
        """
        add Hamiltonian term which is not pre_defined.
        """
        if name in self.hamiltonians:
            raise ValueError(
                'Hamiltonian name %s already defined. The defined names are %s'
                % (name, self.hamiltonians.keys()))
        else:
            self.hamiltonians[name] = Hamiltonian_term

    #@profile
    def get_effective_field(self, S):
        """
        calculate the effective field Heff=-1/ms * \partial H / \partial S
        Langevin term not included.
        """
        Heff = 0.0
        for ham in self.hamiltonians.values():
            Heff += ham.eff_field(S)
        return Heff

    def make_supercell(self, sc_matrix=None, supercell_maker=None):
        if supercell_maker is None:
            smaker = SupercellMaker(sc_matrix)
        else:
            smaker = supercell_maker

        sc_cell = smaker.sc_cell(np.array(self.cell))
        sc_pos = np.array(smaker.sc_pos(np.array(self.pos)))
        sc_zion = smaker.sc_trans_invariant(np.array(self.zion))

        if self.index_spin is not None:
            sc_index_spin = smaker.sc_trans_invariant(self.index_spin)

        sc_Rlist = np.repeat(smaker.R_sc, self.nspin, axis=0)
        sc_iprim = smaker.sc_trans_invariant(list(range(self.nspin)))

        sc_spinat = np.array(smaker.sc_trans_invariant(self.spinat))

        sc_ham = SpinHamiltonian(cell=sc_cell,
                                 pos=sc_pos,
                                 spinat=sc_spinat,
                                 zion=sc_zion,
                                 Rlist=sc_Rlist,
                                 iprim=sc_iprim)

        sc_gyro_ratio = np.array(smaker.sc_trans_invariant(self.gyro_ratio))
        sc_ham.gyro_ratio = sc_gyro_ratio

        sc_gilbert_damping = np.array(
            smaker.sc_trans_invariant(self.gilbert_damping))
        sc_ham.gilbert_damping = sc_gilbert_damping

        if self.has_external_hfield:
            sc_Hext = smaker.sc_trans_invariant(self.H_ext)
            sc_ham.set_external_hfield(sc_Hext)

        if self.has_uniaxial_anisotropy:
            sc_k1 = smaker.sc_trans_invariant(self.k1)
            sc_k1dir = smaker.sc_trans_invariant(self.k1dir)
            sc_ham.set_uniaxial_mca(sc_k1, np.array(sc_k1dir))

        if self.has_exchange:
            sc_Jdict = smaker.sc_ijR(self.exchange_Jdict,
                                     n_basis=len(self.pos))
            sc_ham.set_exchange_ijR(exchange_Jdict=sc_Jdict)

        if self.has_dmi:
            sc_dmi_ddict = smaker.sc_ijR(self.dmi_ddict, n_basis=len(self.pos))
            sc_ham.set_dmi_ijR(sc_dmi_ddict)

        if self.has_bilinear:
            sc_bilinear_dict = smaker.sc_ijR(self.bilinear_dict,
                                             n_basis=len(self.pos))
            sc_ham.set_bilinear_ijR(sc_bilinear_dict)

        return sc_ham

    def calc_total_HijR(self):
        self._total_hessian_ijR = defaultdict(lambda: np.zeros(
            (3, 3), dtype=float))
        for tname, term in self.hamiltonians.items():
            if term.is_twobody_term():
                for key, val in term.hessian_ijR().items():
                    self._total_hessian_ijR[key] += val
        return self._total_hessian_ijR

    def get_total_hessian_ijR(self):
        if self._total_hessian_ijR is None:
            self.calc_total_HijR()
        return self._total_hessian_ijR

    def solve_k(self, kpts, Jq=False):
        """
        Get the eigenvalues and eigenvectors for the kpoints
        """
        qsolver = QSolver(hamiltonian=self)
        evals, evecs = qsolver.solve_all(kpts, eigen_vectors=True, Jq=Jq)
        return evals, evecs

    def find_ground_state_from_kmesh(self, kmesh, myfile):
        kpts = monkhorst_pack(kmesh)
        evals, evecs = self.solve_k(kpts, Jq=True)
        #write_magnon_info(self, kpts, evals, evecs, myfile)

    def plot_magnon_band(self,
                         kvectors=np.array([[0, 0, 0], [0.5, 0, 0],
                                            [0.5, 0.5, 0], [0, 0, 0],
                                            [.5, .5, .5]]),
                         knames=['$\Gamma$', 'X', 'M', '$\Gamma$', 'R'],
                         supercell_matrix=None,
                         npoints=100,
                         color='red',
                         kpath_fname=None,
                         Jq=False,
                         ax=None,
                         ):
        if ax is None:
            fig, ax = plt.subplots()
        if knames is None and kvectors is None:
            # fully automatic k-path
            bp = Cell(self.cell).bandpath(npoints=npoints)
            spk = bp.special_points
            xlist, kptlist, Xs, knames=group_band_path(bp)
        elif knames is not None and kvectors is None:
            # user specified kpath by name
            bp = Cell(self.cell).bandpath(knames, npoints=npoints)
            spk = bp.special_points
            kpts = bp.kpts
            xlist, kptlist, Xs, knames=group_band_path(bp)
        else:
            # user spcified kpath and kvector.
            kpts, x, Xs = bandpath(kvectors, self.cell, npoints)
            spk = dict(zip(knames, kvectors))
            xlist=[x]
            kptlist=[kpts]

        if supercell_matrix is not None:
            kvectors = [np.dot(k, supercell_matrix) for k in kvectors]
        print("High symmetry k-points:")
        for name, k in spk.items():
            if name=='G':
                name='Gamma'
            print(f"{name}: {k}")


        for kpts, xs in zip(kptlist, xlist):
            evals, evecs = self.solve_k(kpts,Jq=Jq)
            # Plot band structure
            nbands = evals.shape[1]
            emin = np.min(evals[:, 0])
            for i in range(nbands):
                ax.plot(xs, (evals[:, i]) / 1.6e-22, color=color)

        ax.set_ylabel('Energy (meV)')
        ax.set_xlim(xlist[0][0], xlist[-1][-1])
        ax.set_xticks(Xs)
        knames=[x if x!='G' else '$\Gamma$' for x in knames]
        ax.set_xticklabels(knames)
        for x in Xs:
            ax.axvline(x, linewidth=0.6, color='gray')
        return ax

    def write_xml(self, fname):
        writer = SpinXmlWriter()
        writer._write(self, fname)


def read_spin_ham_from_file(fname):
    parser = SpinXmlParser(fname)
    ham = SpinHamiltonian(cell=parser.cell,
                          xcart=parser.spin_positions,
                          spinat=parser.spin_spinat,
                          zion=parser.spin_zions)
    ham.set(gilbert_damping=parser.spin_damping_factors,
            gyro_ratio=parser.spin_gyro_ratios)
    if parser.has_exchange:
        ham.set_exchange_ijR(parser.exchange(isotropic=True))
    if parser.has_dmi:
        ham.set_dmi_ijR(parser.dmi)
    if parser.has_bilinear:
        ham.set_bilinear_ijR(parser.bilinear)
    return ham



import numpy as np
from scipy.special import roots_legendre

class Contour():
    def __init__(self, emin, emax=0.0):
        self.emin=emin
        self.emax=emax

    def build_path_semicircle(self, npoints, endpoint=True):
        R= (self.emax-self.emin)/2.0
        R0= (self.emin+self.emax)/2.0
        phi=np.linspace(np.pi, 0, num=npoints+1, endpoint=endpoint)
        p=R0+R*np.exp(1.0j * phi)
        if endpoint:
            self.path=p
            self.de=np.diff(p)
        else:
            self.path=(p[:-1]+p[1:])/2
            self.de=p[1:]-p[:-1]
    
    def build_path_legendre(self, npoints, endpoint=True):
        p=13
        x,w= roots_legendre(npoints)
        R= (self.emax-self.emin)/2.0
        R0= (self.emin+self.emax)/2.0
        y1 = -np.log(1+np.pi*p)
        y2 = 0
        y   = (y2-y1)/2*x+(y2+y1)/2
        phi = (np.exp(-y)-1)/p
        path  = R0+R*np.exp(1.0j*phi)
        #weight= -(y2-y1)/2*np.exp(-y)/p*1j*(path-R0)*w
        if endpoint:
            self.path=path
            self.de=np.diff(path)
        else:
            self.path=(path[:-1]+path[1:])/2
            self.de=path[1:]-path[:-1]
 
    def build_path_rectangle(self, height=0.1, nz1=50, nz2=200, nz3=50):
        """
        prepare list of energy for integration.
        The path has three segments:
         emin --1-> emin + 1j*height --2-> emax+1j*height --3-> emax
        """
        nz1, nz2, nz3 = nz1, nz2, nz3
        nz = nz1+nz2+nz3
        p= np.zeros(nz + 1, dtype='complex128')
        p[:nz1] = self.emin + np.linspace(
            0, height, nz1, endpoint=False) * 1j
        p[nz1:nz1 + nz2] = self.emin + height * 1j + np.linspace(
            0, self.emax - self.emin, nz2, endpoint=False)
        p[nz1 + nz2:nz] = self.emax + height * 1j + np.linspace(
            0, -height, nz3, endpoint=False) * 1j
        p[-1] = self.emax  # emax
        self.path=(p[:-1]+p[1:])/2
        self.de=p[1:]-p[:-1]


    @property
    def npoints(self):
        return len(self.path)

    def de(self):
        return self.de

    def plot(self, ax=None):
        import matplotlib.pyplot as plt
        if ax is None:
            fig,ax=plt.subplots()
        plt.plot(self.path.real, self.path.imag, marker='.')
        plt.show()



def test():
    ct=Contour(emin=-16, emax=0)
    #ct.build_path_semicircle(npoints=100)
    #ct.build_path_rectangle()
    ct.build_path_legendre(npoints=50)
    print(ct.npoints)
    ct.plot()
if __name__ == '__main__':
    test()



"""
A helper class for building supercells
"""
import numpy as np
from collections import OrderedDict
from itertools import product
from functools import lru_cache


class SupercellMaker(object):
    def __init__(self, sc_matrix):
        """
        a helper class for making supercells.
        sc_matrix, supercell matrix. sc_matrix .dot. unitcell = supercell
        """
        sc_matrix = np.array(sc_matrix, dtype=int)
        if len(sc_matrix.flatten()) == 3:
            sc_matrix = np.diag(sc_matrix)
        elif sc_matrix.shape == (3, 3):
            pass
        else:
            raise ValueError('sc_matrix should be 3 or 3*3 matrix')
        self.sc_matrix = sc_matrix
        self.inv_scmat = np.linalg.inv(self.sc_matrix.T)
        self.build_sc_vec()
        self.ncell = int(round(abs(np.linalg.det(sc_matrix))))

    def to_red_sc(self, x):
        return np.dot(self.inv_scmat, x)

    def rotate_vector(self, vec):
        """
        a vector in the new axis.
        """

    def build_sc_vec(self):
        eps_shift = np.sqrt(
            2.0) * 1.0E-8  # shift of the grid, so to avoid double counting
        #max_R = np.max(np.abs(self.sc_matrix)) * 3
        sc_vec = []
        newcell = self.sc_matrix
        scorners_newcell = np.array([[0., 0., 0.], [0., 0., 1.], [0., 1., 0.],
                                     [0., 1., 1.], [1., 0., 0.], [1., 0., 1.],
                                     [1., 1., 0.], [1., 1., 1.]])
        corners = np.dot(scorners_newcell, newcell)
        scorners = corners
        rep = np.ceil(scorners.ptp(axis=0)).astype('int') + 1

        # sc_vec: supercell vector (map atom from unit cell to supercell)
        for vec in product(range(rep[0]), range(rep[1]), range(rep[2])):
            # compute reduced coordinates of this candidate vector in the super-cell frame
            tmp_red = self.to_red_sc(vec)
            # check if in the interior
            if (not (tmp_red <= -1.0 * eps_shift).any()) and (
                    not (tmp_red > 1.0 - eps_shift).any()):
                sc_vec.append(np.array(vec))

        # number of times unit cell is repeated in the super-cell
        num_sc = len(sc_vec)

        # check that found enough super-cell vectors
        if int(round(np.abs(np.linalg.det(self.sc_matrix)))) != num_sc:
            raise Exception(
                "\n\nSuper-cell generation failed! Wrong number of super-cell vectors found."
            )

        self.sc_vec = sc_vec

    def get_R(self):
        return self.sc_vec

    @property
    def R_sc(self):
        return self.sc_vec

    def build_sc_vec2(self):
        """
        TODO rewrite this to reduce memory usage.
        """
        max_R = np.max(np.abs(self.sc_matrix)) * 3
        sc_cands = []
        for i in range(-max_R, max_R + 1):
            for j in range(-max_R, max_R + 1):
                for k in range(-max_R, max_R + 1):
                    sc_cands.append(np.array([i, j, k]))

        # sc_vec: supercell vector (map atom from unit cell to supercell)
        sc_vec = []
        eps_shift = np.sqrt(
            2.0) * 1.0E-8  # shift of the grid, so to avoid double counting
        for vec in sc_cands:
            # compute reduced coordinates of this candidate vector in the super-cell frame
            tmp_red = self.to_red_sc(vec)
            # check if in the interior
            inside = True
            for t in tmp_red:
                if t <= -1.0 * eps_shift or t > 1.0 - eps_shift:
                    inside = False
            if inside:
                sc_vec.append(np.array(vec))
        # number of times unit cell is repeated in the super-cell
        num_sc = len(sc_vec)
        # check that found enough super-cell vectors
        if int(round(np.abs(np.linalg.det(self.sc_matrix)))) != num_sc:
            raise Exception(
                "\n\nSuper-cell generation failed! Wrong number of super-cell vectors found."
            )

        self.sc_vec = sc_vec

    def sc_cell(self, cell):
        cell = np.array(cell)
        if len(cell.flatten()) == 3:
            cell = np.diag(cell)
        return np.dot(self.sc_matrix, cell)

    def sc_pos(self, positions, return_R=False):
        """
        pos -> pos in supercell (reduced.)
        """
        sc_pos = []
        sc_R = []
        for cur_sc_vec in self.sc_vec:  # go over all super-cell vectors
            for pos in positions:
                # shift orbital and compute coordinates in
                # reduced coordinates of super-cell
                sc_pos.append(self.to_red_sc(pos + cur_sc_vec))
                sc_R.append(cur_sc_vec)
        if return_R:
            return sc_pos, sc_R
        else:
            return sc_pos

    def sc_trans_invariant(self, q, return_R=False):
        """
        translation invariant quantities. Like on-site energy of tight binding,
        chemical symbols, magnetic moments of spin.
        """
        sc_q = []
        sc_R = []  # supercell R
        for cur_sc_vec in self.sc_vec:  # go over all super-cell vectors
            for qitem in q:
                sc_q.append(qitem)
                sc_R.append(cur_sc_vec)
        if return_R:
            return sc_q, sc_R
        else:
            return sc_q

    def sc_index(self, indices, n_ind=None):
        """
        Note that the number of indices could be inequal to the repeat period.
        e.g. for n_orb of orbitals, the indices of atoms iatom for each orbital.
        In that case, in the second unit cell (c=1 here), iatom-> iatom+n_ind,
        where n_ind=natoms in primitive cell.
        """
        sc_ind = []
        if n_ind is None:
            n_ind = len(indices)
        for c, cur_sc_vec in enumerate(
                self.sc_vec):  # go over all super-cell vectors
            for ind in indices:
                sc_ind.append(ind + c * n_ind)
        return sc_ind

    @lru_cache(maxsize=10000)
    def _sc_R_to_pair_ind(self, R_plus_Rv):
        """
        R: initial R vector (e.g. R in (i, jR) pair)
        Rv: translation vector. (e.g. for Rv in self.sc_vec)

        Returns:
        sc_part: R in
        pair_ind:
        """
        R_plus_Rv = np.array(R_plus_Rv)
        sc_part = np.floor(self.to_red_sc(R_plus_Rv))  # round down!
        sc_part = np.array(sc_part, dtype=int)
        # find remaining vector in the original reduced coordinates
        orig_part = R_plus_Rv - np.dot(sc_part, self.sc_matrix)
        # remaining vector must equal one of the super-cell vectors
        pair_ind = None
        for p, pair_sc_vec in enumerate(self.sc_vec):
            if False not in (pair_sc_vec == orig_part):
                if pair_ind is not None:
                    raise Exception("\n\nFound duplicate super cell vector!")
                pair_ind = p
        if pair_ind is None:
            raise Exception("\n\nDid not find super cell vector!")
        return sc_part, pair_ind

    @lru_cache(maxsize=10000)
    def sc_jR_to_scjR(self, j, R, Rv, n_basis):
        """
        (j, R) in primitive cell to (j', R') in supercell.
        """
        Rprim, pair_ind = self._sc_R_to_pair_ind(
            tuple(np.array(R) + np.array(Rv)))
        return j + pair_ind * n_basis, tuple(Rprim)

    @lru_cache(maxsize=10000)
    def sc_i_to_sci(self, i, ind_Rv, n_basis):
        return i + ind_Rv * n_basis

    def sc_ijR_only(self, i, j, R, n_basis):
        ret = []
        for c, cur_sc_vec in enumerate(
                self.sc_vec):  # go over all super-cell vectors
            sc_part, pair_ind = self._sc_R_to_pair_ind(
                tuple(R + cur_sc_vec))
            sc_i = i + c * n_basis
            sc_j = j + pair_ind * n_basis
            ret.append((sc_i, sc_j, tuple(sc_part)))
        return ret


    def sc_jR(self, jlist, Rjlist, n_basis):
        sc_jlist=[]
        sc_Rjlist=[]
        for c, cur_sc_vec in enumerate(
                self.sc_vec):  # go over all super-cell vectors
            #for i , j, ind_R, val in
            for j, Rj in zip(jlist, Rjlist):
                sc_part, pair_ind = self._sc_R_to_pair_ind(
                    tuple(Rj + cur_sc_vec))
                sc_j = j + pair_ind * n_basis
                #ret_dict[(sc_i, sc_j, tuple(sc_part))] = val
                sc_jlist.append(sc_j)
                sc_Rjlist.append(tuple(sc_part))
        return sc_jlist, sc_Rjlist



    def sc_ijR(self, terms, n_basis):
        """
        # TODO very slow when supercell is large, should improve it.
        map Val(i, j, R) which is a funciton of (R+rj-ri) to supercell.
        e.g. hopping in Tight binding. exchange in heisenberg model,...
        Args:
        ========================
        terms: either list of [i, j, R, val] or  dict{(i,j, R): val}
        pos: reduced positions in the unit cell.
        Returns:
        =======================
        """
        ret_dict = OrderedDict()
        for c, cur_sc_vec in enumerate(
                self.sc_vec):  # go over all super-cell vectors
            #for i , j, ind_R, val in
            for (i, j, ind_R), val in terms.items():
                #sc_part = np.floor(
                # self.to_red_sc(ind_R + cur_sc_vec))  # round down!

                #pair_ind=self._sc_R_to_pair_ind(tuple(ind_R), tuple(cur_sc_vec))
                sc_part, pair_ind = self._sc_R_to_pair_ind(
                    tuple(ind_R + cur_sc_vec))
                # index of "from" and "to" hopping indices
                sc_i = i + c * n_basis
                sc_j = j + pair_ind * n_basis

                # hi = self._hoppings[h][1] + c * self._norb
                # hj = self._hoppings[h][2] + pair_ind * self._norb
                ret_dict[(sc_i, sc_j, tuple(sc_part))] = val
        return ret_dict

    def sc_atoms(self, atoms):
        """
        This function is compatible with ase.build.make_supercell.
        They should produce the same result.
        """
        from ase.atoms import Atoms
        sc_cell = self.sc_cell(atoms.get_cell())
        sc_pos = self.sc_pos(atoms.get_scaled_positions())
        sc_numbers = self.sc_trans_invariant(atoms.get_atomic_numbers())
        sc_magmoms = self.sc_trans_invariant(
            atoms.get_initial_magnetic_moments())
        return Atoms(
            cell=sc_cell,
            scaled_positions=sc_pos,
            numbers=sc_numbers,
            magmoms=sc_magmoms)

    def phase(self, qpoint):
        return np.exp(2j*np.pi* np.einsum('i, ji -> j', qpoint, self.sc_vec))


def smod(x):
    x = np.mod(x, 1)
    return x if x < 0.5 else x - 1


smod = np.vectorize(smod)


def map_to_primitive(atoms, primitive_atoms, offset=(0, 0, 0)):
    ilist = []
    Rlist = []
    offset = np.array(offset, dtype=float)
    ppos = primitive_atoms.get_positions()
    pos = atoms.get_positions()
    cell = primitive_atoms.get_cell()
    for p in pos:
        found = False
        for i, pp in enumerate(ppos):
            res0 = np.linalg.solve(cell, p - pp)
            res = smod(res0)
            if np.linalg.norm(res) < 0.01:
                found = True
                R = res0 - res
                ilist.append(i)
                Rlist.append(R)
        if not found:
            print("Not found")
    return np.array(ilist, dtype=int), np.array(Rlist, dtype=int)


def test():
    sc_mat = np.diag([1, 1, 2])
    #sc_mat[0, 1] = 2
    spm = supercell_maker(sc_matrix=sc_mat)
    print(spm.sc_cell([1, 1, 1]))
    print(spm.sc_pos([[0.5, 1, 1]]))
    print(spm.sc_trans_invariant(['Fe']))
    print(
        spm.sc_ijR({
            (0, 0, (0, 0, 1)): 1.2,
            (1, 1, (0, 0, 1)): 1.2,
        }, [(0, 0, 0)]))
    print(spm.sc_index(indices=(1, 2)))
    print(spm.sc_index(indices=(1, 2), n_ind=4))
    from ase.atoms import Atoms
    atoms = Atoms('HO', positions=[[0, 0, 0], [0, 0.2, 0]], cell=[1, 1, 1])
    from ase.build import make_supercell
    atoms2 = make_supercell(atoms, sc_mat)
    atoms3 = spm.sc_atoms(atoms)
    #print(atoms2.get_positions())
    #print(atoms3.get_positions())
    assert (atoms2 == atoms3)
    assert (atoms2.get_positions() == atoms3.get_positions()).all()


if __name__ == '__main__':
    test()



#!/usr/bin/env python
from __future__ import division
import ase
from ase.atoms import Atoms
from mpl_toolkits.mplot3d import axes3d
import matplotlib.pyplot as plt
import numpy as np
import numpy.linalg
from ase.geometry import cell_to_cellpar, cellpar_to_cell
from ase.dft.kpoints import (get_special_points, bandpath, special_paths,
                             parse_path_string)
#from minimulti.spin.hamiltonian import SpinHamiltonian
#from minimulti.spin.mover import SpinMover
from .qsolver import QSolver
from .constants import mu_B, meV



def group_band_path(bp, eps=1e-8, shift=0.15):
    xs, Xs, knames = bp.get_linear_kpoint_axis()
    kpts = bp.kpts

    m = (xs[1:]-xs[:-1] < eps)
    segments = [0]+list(np.where(m)[0]+1)+[len(xs)]

    # split Xlist
    xlist, kptlist = [], []
    for i, (start, end) in enumerate(zip(segments[:-1], segments[1:])):
        kptlist.append(kpts[start:end])
        xlist.append(xs[start:end]+i*shift)

    m = (Xs[1:]-Xs[:-1] < eps)

    s = np.where(m)[0] + 1

    for i in s:
        Xs[i:] += shift

    return xlist, kptlist, Xs, knames


def test_group_band_path():
    atoms = ase.Atoms('H', cell=[1, 1, 2])
    bp = atoms.cell.bandpath(npoints=50)
    xlist, kptlist, Xs, knames = group_band_path(bp)

    for x, k in zip(xlist, kptlist):
        plt.plot(x,x)

    plt.xticks(Xs, knames)
    plt.show()



if __name__=="__main__":
    test_group_band_path()


def plot_3d_vector(positions, vectors, length=0.1):
    fig = plt.figure()
    ax = fig.gca(projection='3d')
    n = positions.shape[1]
    x, y, z = positions.T
    u, v, w = vectors.T
    ax.quiver(
        x,
        y,
        z,
        u,
        v,
        w,
        length=length,
        normalize=False,
        pivot='middle',
        cmap='seismic')
    plt.show()


def plot_2d_vector(positions, vectors, show_z=True, length=0.1, ylimit=None):
    fig = plt.figure()
    ax = fig.add_subplot(111)
    n = positions.shape[1]
    x, y, z = positions.T
    u, v, w = vectors.T
    #ax.streamplot(x, y, u, v,  linewidth=1, cmap=plt.cm.inferno,
    #    density=2, arrowstyle='->', arrowsize=1.5)
    ax.scatter(x, y, s=50, color='r')
    if show_z:
        ax.quiver(
            x,
            y,
            u,
            v,
            w,
            length=length,
            units='width',
            pivot='middle',
            cmap='seismic',
        )
    else:
        ax.quiver(x, y, u, v, units='width', pivot='middle', cmap='seismic')
    if ylimit is not None:
        ax.set_ylim(ylimit[0], ylimit[1])
    #plt.colorbar()
    plt.xlabel('x')
    plt.ylabel('y')
    plt.show()


def plot_supercell(ham,
                   supercell_matrix=np.diag([30, 1, 1]),
                   plot_type='2d',
                   length=0.1,
                   ylimit=None):
    sc_ham = ham.make_supercell(supercell_matrix)
    sc_ham.s = np.random.rand(*sc_ham.s.shape) - 0.5
    mover = SpinMover(hamiltonian=sc_ham)
    mover.set(time_step=3e-4, temperature=0, total_time=6, save_all_spin=False)

    mover.run(write_step=20)
    pos = np.dot(sc_ham.pos, sc_ham.cell)
    if plot_type == '2d':
        plot_2d_vector(
            pos, mover.s, show_z=False, length=length, ylimit=ylimit)
    elif plot_type == '3d':
        plot_3d_vector(pos, mover.s, length=length)


#exchange_1d()

from ase.dft.kpoints import *
def mybandpath(path, cell, npoints=50, eps=1e-3):
    """Make a list of kpoints defining the path between the given points.

    path: list or str
        Can be:

        * a string that parse_path_string() understands: 'GXL'
        * a list of BZ points: [(0, 0, 0), (0.5, 0, 0)]
        * or several lists of BZ points if the the path is not continuous.
    cell: 3x3
        Unit cell of the atoms.
    npoints: int
        Length of the output kpts list.

    Return list of k-points, list of x-coordinates and list of
    x-coordinates of special points."""

    if isinstance(path, str):
        special = get_special_points(cell,eps=eps)
        paths = []
        for names in parse_path_string(path):
            paths.append([special[name] for name in names])
    elif np.array(path[0]).ndim == 1:
        paths = [path]
    else:
        paths = path

    points = np.concatenate(paths)
    dists = points[1:] - points[:-1]
    lengths = [np.linalg.norm(d) for d in kpoint_convert(cell, skpts_kc=dists)]

    i = 0
    for path in paths[:-1]:
        i += len(path)
        lengths[i - 1] = 0

    length = sum(lengths)
    kpts = []
    x0 = 0
    x = []
    X = [0]
    for P, d, L in zip(points[:-1], dists, lengths):
        n = max(2, int(round(L * (npoints - len(x)) / (length - x0))))

        for t in np.linspace(0, 1, n)[:-1]:
            kpts.append(P + t * d)
            x.append(x0 + t * L)
        x0 += L
        X.append(x0)
    kpts.append(points[-1])
    x.append(x0)

    return np.array(kpts), np.array(x), np.array(X)

def fix_cell(cell,eps=5e-3):
    cellpar=cell_to_cellpar(cell)
    for i in [3,4,5]:
        if abs(cellpar[i]/90.0*np.pi/2-np.pi/2)<eps:
            cellpar[i]=90.0
    return cellpar_to_cell(cellpar)

def plot_M_vs_time(ham, supercell_matrix=np.eye(3), temperature=0):
    sc_ham = ham.make_supercell(supercell_matrix)
    mover = SpinMover(hamiltonian=sc_ham)
    mover.set(
        time_step=1e-5,
        temperature=temperature,
        total_time=1,
        save_all_spin=True)

    mover.run(write_step=10)

    hist = mover.get_hist()
    hspin = np.array(hist['spin'])
    time = np.array(hist['time'])
    tspin = np.array(hist['total_spin'])

    Ms = np.linalg.det(supercell_matrix)
    plt.figure()
    plt.plot(
        time, np.linalg.norm(tspin, axis=1) / Ms, label='total', color='black')
    plt.plot(time, tspin[:, 0] / Ms, label='x')
    plt.plot(time, tspin[:, 1] / Ms, label='y')
    plt.plot(time, tspin[:, 2] / Ms, label='z')
    plt.xlabel('time (s)')
    plt.ylabel('magnetic moment ($\mu_B$)')
    plt.legend()
    #plt.show()
    #avg_total_m = np.average((np.linalg.norm(tspin, axis=1)/Ms)[:])
    plt.show()


def plot_M_vs_T(ham, supercell_matrix=np.eye(3), Tlist=np.arange(0.0, 110,
                                                                 20)):
    Mlist = []
    for temperature in Tlist:
        sc_ham = ham.make_supercell(supercell_matrix)
        mover = SpinMover(hamiltonian=sc_ham)
        mover.set(
            time_step=2e-4,
            #damping_factor=0.1,
            temperature=temperature,
            total_time=1,
            save_all_spin=True)

        mover.run(write_step=10)

        hist = mover.get_hist()
        hspin = np.array(hist['spin'])
        time = np.array(hist['time'])
        tspin = np.array(hist['total_spin'])

        Ms = np.linalg.det(supercell_matrix)
        avg_total_m = np.average((np.linalg.norm(tspin, axis=1) / Ms)[300:])
        print("T: %s   M: %s" % (temperature, avg_total_m))
        Mlist.append(avg_total_m)

    plt.plot(Tlist, Mlist)
    plt.ylim(-0.01, 1.01)
    plt.xlabel('Temperature (K)')
    plt.ylabel('Average magnetization (Ms)')
    plt.show()



#!/usr/bin/env python3
import argparse
import sys
from TB2J.manager import gen_exchange
from TB2J.versioninfo import print_license


def run_wann2J():
    print_license()
    parser = argparse.ArgumentParser(
        description=
        "wann2J: Using magnetic force theorem to calculate exchange parameter J from wannier functions"
    )
    parser.add_argument('--path',
                        help="path to the wannier files",
                        default='./',
                        type=str)
    parser.add_argument('--posfile',
                        help="name of the position file",
                        default='POSCAR',
                        type=str)
    parser.add_argument('--prefix_spinor',
                        help="prefix to the spinor wannier files",
                        default='wannier90',
                        type=str)
    parser.add_argument('--prefix_up',
                        help="prefix to the spin up wannier files",
                        default='wannier90.up',
                        type=str)
    parser.add_argument('--prefix_down',
                        help="prefix to the spin down wannier files",
                        default='wannier90.dn',
                        type=str)
    parser.add_argument('--elements',
                        help="elements to be considered in Heisenberg model",
                        default=None,
                        type=str,
                        nargs='*')
    parser.add_argument(
        '--groupby',
        help=
        "In the spinor case, the order of the orbitals have two conventions: 1: group by spin (orb1_up, orb2_up,... orb1_down, ...), 2,group by orbital (orb1_up, orb1_down, orb2_up, ...,). Use 'spin' in the former case and 'orbital' in the latter case. The default is spin.",
        default='spin',
        type=str,
    )

    parser.add_argument(
        '--rcut',
        help=
        'cutoff of spin pair distance. The default is to calculate all commensurate R point to the k mesh.',
        default=None,
        type=float)
    parser.add_argument('--efermi',
                        help='Fermi energy in eV',
                        default=None,
                        type=float)
    parser.add_argument('--kmesh',
                        help='kmesh in the format of kx ky kz',
                        type=int,
                        nargs='*',
                        default=[5, 5, 5])
    parser.add_argument('--emin',
                        help='energy minimum below efermi, default -14 eV',
                        type=float,
                        default=-14.0)
    parser.add_argument('--emax',
                        help='energy maximum above efermi, default 0.0 eV',
                        type=float,
                        default=0.0)
    parser.add_argument(
        '--nz',
        help='number of steps for semicircle contour, default: 100',
        default=100,
        type=int)
    parser.add_argument(
        '--height',
        help=
        'energy contour, a small number (often between 0.1 to 0.5, default 0.1)',
        type=float,
        default=0.1)
    parser.add_argument('--nz1',
                        help='number of steps 1, default: 50',
                        default=50,
                        type=int)
    parser.add_argument('--nz2',
                        help='number of steps 2, default: 200',
                        default=200,
                        type=int)
    parser.add_argument('--nz3',
                        help='number of steps 3, default: 50',
                        default=50,
                        type=int)
    parser.add_argument(
        '--cutoff',
        help="The minimum of J amplitude to write, (in eV), default is 1e-5 eV",
        default=1e-5,
        type=float)
    parser.add_argument(
        '--exclude_orbs',
        help=
        "the indices of wannier functions to be excluded from magnetic site. counting start from 0",
        default=[],
        type=int,
        nargs='+')

    parser.add_argument('--np',
                        help='number of cpu cores to use in parallel, default: 1',
                        default=1,
                        type=int)


    parser.add_argument(
        '--use_cache',
        help=
        "whether to use disk file for temporary storing wavefunctions and hamiltonian to reduce memory usage. Default: False",
        action='store_true',
        default=False)

    parser.add_argument(
        "--description",
        help=
        "add description of the calculatiion to the xml file. Essential information, like the xc functional, U values, magnetic state should be given.",
        type=str,
        default="Calculated with TB2J.")

    parser.add_argument("--spinor",
                        action="store_true",
                        help="Whether to use spinor wannier function.",
                        default=False)

    parser.add_argument(
        "--output_path",
        help="The path of the output directory, default is TB2J_results",
        type=str,
        default="TB2J_results")

    parser.add_argument(
        "--wannier_type",
        help="The type of Wannier function, either Wannier90 or banddownfolder",
        type=str,
        default="Wannier90")

    parser.add_argument("--qspace",
                        action="store_true",
                        help="Whether to calculate J in qspace first and transform to real space.",
                        default=False)


    args = parser.parse_args()

    if args.efermi is None:
        print("Please input fermi energy using --efermi ")
        sys.exit()
    if args.elements is None:
        print("Please input the magnetic elements, e.g. --elements Fe Ni")
        sys.exit()

    gen_exchange(path=args.path,
                 colinear=(not args.spinor),
                 groupby=args.groupby,
                 posfile=args.posfile,
                 efermi=args.efermi,
                 kmesh=args.kmesh,
                 magnetic_elements=args.elements,
                 Rcut=args.rcut,
                 prefix_SOC=args.prefix_spinor,
                 prefix_up=args.prefix_up,
                 prefix_dn=args.prefix_down,
                 emin=args.emin,
                 emax=args.emax,
                 nz=args.nz,
                 height=args.height,
                 nz1=args.nz1,
                 nz2=args.nz2,
                 nz3=args.nz3,
                 use_cache=args.use_cache,
                 np=args.np,
                 description=args.description,
                 output_path=args.output_path,
                 exclude_orbs=args.exclude_orbs, 
                 wannier_type=args.wannier_type,
                 qspace= args.qspace
            )


if __name__ == "__main__":
    run_wann2J()



"""
IO module. called io_xml for historical reason.
write not only xml output.
- xml
- multibinit input template
- vampire inputs
- uppasd inputs (not tested yet)
- tom's asd inputs.
"""

import os
from collections.abc import Iterable
import numpy as np
from ase.dft.kpoints import monkhorst_pack
import pickle
from ase.cell import Cell
from TB2J import __version__
from datetime import datetime
import matplotlib.pyplot as plt


class SpinIO(object):
    def __init__(
        self,
        atoms,
        spinat,
        charges,
        index_spin,
        orbital_names={},
        colinear=True,
        distance_dict=None,
        exchange_Jdict=None,
        exchange_Jdict_orb=None,
        dJdx=None,
        dmi_ddict=None,
        Jani_dict=None,
        biquadratic_Jdict=None,
        debug_dict=None,
        k1=None,
        k1dir=None,
        NJT_Jdict=None,
        NJT_ddict=None,
        damping=None,
        gyro_ratio=None,
        write_experimental=True,
        description=None,
    ):
        """
        :param atoms: Ase atoms structure.
        :param spinat: spin for each atom. (3*natom)
        :param charges: charges for each atom (natom)
        :param index_spin: index of spin in the spin potential for each atom.
        :param colinear: whether the parameters are for a colinear wannier calculation.
        :param distance_dict: {(R, i,j ): distance}
        :param exchange_Jdict: {(R, i,j): J}
        :param exchange_Jdict_orb: {(R, i,j): J}
        :param dJdx: {(R, i,j): dJdx}
        :param dim_ddict:{(R, i,j): DMI}
        :param Jani_dict: {(R, i,j): Jani'}, Jani is a 3*3 matrix
        :param biqudratic_Jdict: {(R, i,j ): J}
        :param k1: single ion anisotropy amplitude.o
        :param kdir: directio of k1
        :param NJT_Jdict: exhange calculated using NJT method
        :param NJT_ddict: DMI calculated using NJT method
        :param damping: damping factor
        :param gyro_ratio:  gyromagnetic ratio
        :param write_experimental: write_experimental data to output files
        :param description: add some description into the xml file.
        """
        self.atoms = atoms  #: atomic structures, ase.Atoms object
        self.index_spin = index_spin  #: index of spin linked to atoms. -1 if non-magnetic
        self.spinat = spinat  #: spin for each atom. shape of (natom, 3)
        self.colinear = colinear  #: If the calculation is collinear or not
        if self.colinear and self.spinat != []:
            self.magmoms = np.array(self.spinat)[:, 2]
        self.charges = charges
        #: A dictionary of distances, the keys are (i,j, R), where i and j are spin index and R is the cell index, a tuple of three integers.
        self.distance_dict = distance_dict
        self.ind_atoms = {}  #: The index of atom for each spin.
        for iatom, ispin in enumerate(self.index_spin):
            if ispin >= 0:
                self.ind_atoms[ispin] = iatom

        if exchange_Jdict is not None:
            self.has_exchange = True  #: whether there is isotropic exchange
            #: The dictionary of :math:`J_{ij}(R)`, the keys are (i,j, R), where R is a tuple, and the value is the isotropic exchange
            self.exchange_Jdict = exchange_Jdict
        else:
            self.has_exchange = False
            self.exchange_Jdict = None

        self.exchange_Jdict_orb = exchange_Jdict_orb

        self.dJdx = dJdx

        if dmi_ddict is not None:
            self.has_dmi = True  #: Whether there is DMI.
            #: The dictionary of DMI. the key is the same as exchange_Jdict, the values are 3-d vectors (Dx, Dy, Dz).
            self.dmi_ddict = dmi_ddict
        else:
            self.has_dmi = False
            self.dmi_ddict = None

        if Jani_dict is not None:
            self.has_bilinear = True  #: Whether there is anisotropic exchange term
            #: The dictionary of anisotropic exchange. The vlaues are matrices of shape (3,3).
            self.Jani_dict = Jani_dict
        else:
            self.has_bilinear = False
            self.Jani_dict = None

        if k1 is not None and k1dir is not None:
            self.has_uniaxial_anistropy = True
            self.k1 = k1
            self.k1dir = k1dir
        else:
            self.has_uniaxial_anistropy = False
            self.k1 = None
            self.k1dir = None

        self.has_bilinear = not (Jani_dict == {} or Jani_dict is None)

        self.has_biquadratic = not (biquadratic_Jdict == {}
                                    or biquadratic_Jdict is None)
        self.biquadratic_Jdict = biquadratic_Jdict

        if NJT_ddict is not None:
            self.has_NJT_dmi = True
        self.NJT_ddict = NJT_ddict

        if NJT_Jdict is not None:
            self.has_NJT_exchange = True
        self.NJT_Jdict = NJT_Jdict

        natom = len(self.atoms)
        if gyro_ratio is None:
            self.gyro_ratio = [1.0
                               ] * natom  #: Gyromagnetic ratio for each atom
        elif isinstance(gyro_ratio, Iterable):
            self.gyro_ratio = gyro_ratio
        else:
            self.gyro_ratio = [gyro_ratio] * natom

        if damping is None:
            self.damping = [1.0] * natom  # damping factor for each atom
        elif isinstance(damping, Iterable):
            self.damping = damping
        else:
            self.damping = [damping] * natom

        self.debug_dict = debug_dict

        self.write_experimental = write_experimental

        now = datetime.now()
        self.description = f"""Exchange parameters generated by TB2J {__version__}
Generation time: {now.strftime("%y/%m/%d %H:%M:%S")}
"""
        if description is not None:
            self.description += description

        self.orbital_names = orbital_names

    def get_J(self, i,j, R):
        return self.exchange_Jdict[(tuple(R), i,j)]

    def write_pickle(self, path='TB2J_results', fname='TB2J.pickle'):
        if not os.path.exists(path):
            os.makedirs(path)
        fname = os.path.join(path, fname)
        with open(fname, 'wb') as myfile:
            try:
                pickle.dump(self.__dict__, myfile)
            except Exception as ex:
                print(f"Pickle not written due to {ex}")

    @classmethod
    def load_pickle(cls, path='TB2J_resutls', fname='TB2J.pickle'):
        fname = os.path.join(path, fname)
        with open(fname, 'rb') as myfile:
            d = pickle.load(myfile)
        obj = cls(atoms=[], spinat=[], charges=[], index_spin=[])
        obj.__dict__.update(d)
        return obj

    def write_all(self, path='TB2J_results'):
        self.write_pickle(path=path)
        self.write_txt(path=path)
        self.write_multibinit(path=os.path.join(path, 'Multibinit'))
        self.write_tom_format(path=os.path.join(path, 'TomASD'))
        self.write_vampire(path=os.path.join(path, 'Vampire'))
        self.plot_all(savefile=os.path.join(path, 'JvsR.pdf'))
        self.write_Jq(kmesh=[9, 9, 9], path=path)

    def write_txt(self, path):
        from TB2J.io_exchange.io_txt import write_txt
        write_txt(self, path=path, write_experimental=self.write_experimental)

    def write_multibinit(self, path):
        from TB2J.io_exchange.io_multibinit import write_multibinit
        write_multibinit(self, path=path)

    def write_Jq(self, kmesh, path, **kwargs):
        from TB2J.spinham.spin_api import SpinModel
        from TB2J.io_exchange.io_txt import write_Jq_info
        m = SpinModel(fname=os.path.join(path, 'Multibinit', 'exchange.xml'))
        m.set_ham(**kwargs)
        kpts1 = monkhorst_pack(kmesh)
        bp = Cell(self.atoms.cell).bandpath(npoints=400)
        kpts2 = bp.kpts
        kpts = np.vstack([kpts1, kpts2])

        evals, evecs = m.ham.solve_k(kpts, Jq=True)
        with open(os.path.join(path, 'summary.txt'), 'w') as myfile:
            myfile.write("=" * 60)
            myfile.write("\n")
            myfile.write("Generated by TB2J %s.\n" % (__version__))
            myfile.write("=" * 60 + '\n')
            myfile.write(
                "The spin ground state is estimated by calculating\n the eigenvalues and eigen vectors of J(q):\n"
            )
            write_Jq_info(self, kpts, evals, evecs, myfile, special_kpoints={})

    def plot_JvsR(self,
                  ax=None,
                  color='blue',
                  marker='o',
                  fname=None,
                  show=False,
                  **kwargs,
                  ):
        if ax is None:
            fig, ax = plt.subplots()
        ds = []
        Js = []
        for key, val in self.exchange_Jdict.items():
            d = self.distance_dict[key][1]
            ds.append(d)
            Js.append(val * 1e3)
        ax.scatter(ds, Js, marker=marker, color=color, **kwargs)
        ax.axhline(color='gray')
        ax.set_xlabel("Distance ($\AA$)")
        ax.set_ylabel("J (meV)")
        if fname is not None:
            plt.savefig(fname)
        if show:
            plt.show()
        return ax

    def plot_DvsR(self, ax=None, fname=None, show=False):
        if ax is None:
            fig, ax = plt.subplots()
        ds = []
        Ds = []
        for key, val in self.dmi_ddict.items():
            d = self.distance_dict[key][1]
            ds.append(d)
            Ds.append(val * 1e3)
        Ds = np.array(Ds)
        ax.scatter(ds, Ds[:, 0], marker='s', color='r', label="Dx")
        ax.scatter(ds,
                   Ds[:, 1],
                   marker='o',
                   edgecolors='g',
                   facecolors='none',
                   label='Dy')
        ax.scatter(ds,
                   Ds[:, 2],
                   marker='D',
                   edgecolors='b',
                   facecolors='none',
                   label='Dz')
        ax.axhline(color='gray')
        ax.legend(loc=1)
        ax.set_ylabel("D (meV)")
        if fname is not None:
            plt.savefig(fname)
        if show:
            plt.show()
        return ax

    def plot_JanivsR(self, ax=None, fname=None, show=False):
        if ax is None:
            fig, ax = plt.subplots()
        ds = []
        Jani = []
        for key, val in self.Jani_dict.items():
            d = self.distance_dict[key][1]
            ds.append(d)
            val = val - np.diag([np.trace(val) / 3] * 3)
            Jani.append(val * 1e3)
        Jani = np.array(Jani)
        s = 'xyz'
        for i in range(3):
            ax.scatter(ds, Jani[:, i, i], marker='s', label=f"J{s[i]}{s[i]}")
        c = 'rgb'
        for ic, (i, j) in enumerate([(0, 1), (0, 2), (1, 2)]):
            ax.scatter(ds,
                       Jani[:, i, j],
                       edgecolors=c[ic],
                       facecolors='none',
                       label=f"J{s[i]}{s[j]}")
        ax.axhline(color='gray')
        ax.legend(loc=1, ncol=2)
        ax.set_ylabel("Jani (meV)")
        if fname is not None:
            plt.savefig(fname)
        if show:
            plt.show()
        return ax

    def plot_all(self, title=None, savefile=None, show=False):
        if self.has_dmi and self.has_bilinear:
            naxis = 3
        else:
            naxis = 1
        fig, axes = plt.subplots(naxis, 1, sharex=True, figsize=(5, 2.2*naxis))

        if self.has_dmi and self.has_bilinear:
            self.plot_JvsR(axes[0])
            self.plot_DvsR(axes[1])
            self.plot_JanivsR(axes[2])
        else:
            self.plot_JvsR(axes)

        if title is not None:
            fig.suptitle(title)
        plt.tight_layout()
        plt.subplots_adjust(hspace=0.03, top=0.93)
        if savefile is not None:
            plt.savefig(savefile)
        if show:
            plt.show()
        plt.clf()
        plt.close()
        return fig, axes



    def write_tom_format(self, path):
        from TB2J.io_exchange.io_tomsasd import write_tom_format
        write_tom_format(self, path=path)

    def write_vampire(self, path):
        from TB2J.io_exchange.io_vampire import write_vampire
        write_vampire(self, path=path)

    def write_uppasd(self, path):
        from TB2J.io_exchange.io_uppasd import write_uppasd
        write_uppasd(self, path=path)


def gen_distance_dict(ind_mag_atoms, atoms, Rlist):
    distance_dict = {}
    ind_matoms = ind_mag_atoms
    for ispin, iatom in enumerate(ind_matoms):
        for jspin, jatom in enumerate(ind_matoms):
            for R in Rlist:
                pos_i = atoms.get_positions()[iatom]
                pos_jR = atoms.get_positions()[jatom] + np.dot(
                    R, atoms.get_cell())
                vec = pos_jR - pos_i
                distance = np.sqrt(np.sum(vec**2))
                distance_dict[(tuple(R), ispin, jspin)] = (vec, distance)
    return distance_dict


def test_spin_io():
    from ase import Atoms
    import numpy as np
    atoms = Atoms('SrMnO3',
                  cell=np.eye(3) * 3.8,
                  scaled_positions=[[0, 0, 0], [0.5, 0.5, 0.5], [0, .5, .5],
                                    [.5, 0, .5], [.5, .5, 0]])
    spinat = [[0, 0, x] for x in [0, 3, 0, 0, 0]]
    charges = [2, 4, 5, 5, 5]
    index_spin = [-1, 0, -1, -1, -1]
    colinear = True
    Rlist = [[0, 0, 0], [0, 0, 1]]
    ind_mag_atoms = [1]
    distance_dict = gen_distance_dict(ind_mag_atoms, atoms, Rlist)

    R0 = (0, 0, 0)
    R1 = (0, 0, 1)
    exchange_Jdict = {(R0, 0, 0): 1.2, (R1, 0, 0): 1.1}

    sio = SpinIO(atoms,
                 spinat,
                 charges,
                 index_spin,
                 colinear=True,
                 distance_dict=distance_dict,
                 exchange_Jdict=exchange_Jdict)

    sio.write_all()

if __name__=='__main__':
    test_spin_io()



#!/usr/bin/env python3
import copy
from ase.io import read, write

def rotate_atom_xyz(atoms):
    """
    given a atoms, return:
    - 'z'->'x'
    - 'z'->'y'
    - 'z'->'z'
    """
    atoms_x = copy.deepcopy(atoms)
    atoms_x.rotate(90, 'y', rotate_cell=True)
    atoms_y = copy.deepcopy(atoms)
    atoms_y.rotate(90, 'x', rotate_cell=True)
    atoms_z = atoms
    return atoms_x, atoms_y, atoms_z


def check_ftype(ftype):
    if ftype in ['cif']:
        print("="*40)
        print("WARNING!!!!!")
        print(f'{ftype} type does not contains the cell matrix explicitly. Therefore the outputted files does not give the rotation properly. Please use another format.')
        print("="*40)

def rotate_xyz(fname, ftype='xyz'):
    check_ftype(ftype)
    atoms = read(fname)
    atoms.set_pbc(True)

    atoms_x, atoms_y, atoms_z = rotate_atom_xyz(atoms)

    fname_x = f"atoms_x.{ftype}"
    fname_y = f"atoms_y.{ftype}"
    fname_z = f"atoms_z.{ftype}"

    write(fname_x, atoms_x)
    write(fname_y, atoms_y)
    write(fname_z, atoms_z)
    print(f"The output has been written to {fname_x}, {fname_y}, {fname_z}")


